package be.iminds.ilabt.jfed.experimenter_gui.bugreporting;

import be.iminds.ilabt.jfed.experimenter_gui.ui.SceneAwareController;
import be.iminds.ilabt.jfed.preferences.GuiPreferenceKey;
import be.iminds.ilabt.jfed.preferences.JFedGuiPreferences;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import be.iminds.ilabt.jfed.util.OSDetector;
import javafx.beans.binding.Bindings;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.fxml.FXML;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.image.ImageView;
import javafx.scene.input.Clipboard;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyCodeCombination;
import javafx.scene.layout.StackPane;
import javafx.stage.FileChooser;
import javafx.stage.Stage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.inject.Inject;
import java.io.File;
import java.util.Date;

/**
 * User: twalcari
 * Date: 12/9/13
 * Time: 8:58 AM
 */
public class BugReportDialog implements SceneAwareController {
    private static final Logger LOG = LoggerFactory.getLogger(BugReportDialog.class);
    private static final String BUGREPORT_EXTENSION = ".xml";

    private final BugReportComposer bugReportComposer;
    private final JFedGuiPreferences jFedPreferences;
    private final Clipboard systemClipboard = Clipboard.getSystemClipboard();

    private final BooleanProperty submitting = new SimpleBooleanProperty(false);

    @FXML
    private TextArea descriptionTextArea;
    @FXML
    private TextField versionTextField;
    @FXML
    private TextField environmentTextField;
    @FXML
    private TextField credentialsTextField;
    @FXML
    private TextField emailTextField;
    @FXML
    private Button saveToFileButton;
    @FXML
    private Button submitButton;
    @FXML
    private Button cancelButton;
    @FXML
    private Label includedCallsLabel;
    @FXML
    private ProgressIndicator submitProgressIndicator;
    @FXML
    private Label progressLabel;

    @FXML
    private StackPane screenshotStackPane;
    @FXML
    private CheckBox includeScreenshotCheckBox;
    @FXML
    private ImageView screenshotImageView;
    @FXML
    private Button pasteScreenshotButton;

    private BugReportToFileTask reportTask = null;

    @Inject
    BugReportDialog(BugReportComposer bugReportComposer, JFedGuiPreferences jFedPreferences) {
        this.bugReportComposer = bugReportComposer;
        this.jFedPreferences = jFedPreferences;
    }

    @FXML
    private void initialize() {
        submitButton.disableProperty().bind(submitting);
        saveToFileButton.disableProperty().bind(submitting);
        submitProgressIndicator.visibleProperty().bind(submitting);


        descriptionTextArea.textProperty().bindBidirectional(bugReportComposer.descriptionProperty());
        emailTextField.textProperty().bindBidirectional(bugReportComposer.emailProperty());
        versionTextField.textProperty().bind(bugReportComposer.versionProperty());
        environmentTextField.textProperty().bind(bugReportComposer.environmentProperty());
        credentialsTextField.textProperty().bind(bugReportComposer.credentialsProperty());
        includedCallsLabel.textProperty().bind(Bindings.format("%d calls", bugReportComposer.callsCountProperty()));

        screenshotImageView.imageProperty().bindBidirectional(bugReportComposer.screenshotProperty());
        screenshotImageView.fitWidthProperty().bind(screenshotStackPane.widthProperty());
        screenshotImageView.fitHeightProperty().bind(screenshotStackPane.heightProperty());

        //bind screenshot-imageview to clipboard
        screenshotImageView.imageProperty().addListener(observable -> {
            if (screenshotImageView.getImage() != null) {
                includeScreenshotCheckBox.setSelected(true);
                includeScreenshotCheckBox.setDisable(false);
                pasteScreenshotButton.setVisible(false);
            }
        });

        includeScreenshotCheckBox.setOnAction(observable -> {
            screenshotImageView.setImage(null);
            includeScreenshotCheckBox.setDisable(true);
            pasteScreenshotButton.setVisible(true);
        });
    }

    @FXML
    private void onSaveToFileAction() {
        if (!checkValidity())
            return;

        FileChooser fc = new FileChooser();
        fc.setTitle("Save Bug Report");
        fc.getExtensionFilters().add(new FileChooser.ExtensionFilter("Bug Report (*." + BUGREPORT_EXTENSION + ")", "*" + BUGREPORT_EXTENSION));
        fc.setInitialFileName(String.format("BugReport-%1$tF-%1$tH-%1$tM.xml", new Date()));


        //set the default save dir
        if (System.getProperty("user.home") != null)
            fc.setInitialDirectory(new File(System.getProperty("user.home")));

        if (jFedPreferences.containsPreference(GuiPreferenceKey.PREF_RSPECFOLDER_REQUEST)) {
            File dir = new File(jFedPreferences.getString(GuiPreferenceKey.PREF_RSPECFOLDER_REQUEST));
            if (dir.exists()) {
                fc.setInitialDirectory(dir);
            }
        }

        File file = fc.showSaveDialog(submitButton.getScene().getWindow());

        if (file != null) {
            BugReportToFileTask bugReportToFileTask = new BugReportToFileTask(bugReportComposer, file);
            startAndShowBugReportTask(bugReportToFileTask);
        }

    }


    @FXML
    private void onSubmitAction() {
        if (checkValidity()) {
            BugReportToServerTask bugReportToServerTask = new BugReportToServerTask(bugReportComposer);
            startAndShowBugReportTask(bugReportToServerTask);
        }
    }

    private boolean checkValidity() {
        if (descriptionTextArea == null || descriptionTextArea.getText() == null || descriptionTextArea.getText().isEmpty()) {
            JFDialogs.create()
                    .owner(submitButton.getScene().getWindow())
                    .message("Please fill in a description to submit a bug report.")
                    .masthead("Description is a mandatory field")
                    .showError();
            return false;
        }

        if (emailTextField.getText() == null || emailTextField.getText().isEmpty()) {
            JFDialogs.create()
                    .owner(submitButton.getScene().getWindow())
                    .message("You must provide an email address on which we can contact you")
                    .masthead("Email is a mandatory field")
                    .showError();
            return false;
        }      else if(!isValidEmailAddress(emailTextField.getText())){
            JFDialogs.create()
                    .owner(submitButton.getScene().getWindow())
                    .message("Your email address is invalid. Please check your input!")
                    .masthead("Email is a mandatory field")
                    .showError();
            return false;
        }

        return true;
    }

    private static boolean isValidEmailAddress(String email) {
        String ePattern = "^[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@((\\[[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\])|(([a-zA-Z\\-0-9]+\\.)+[a-zA-Z]{2,}))$";
        java.util.regex.Pattern p = java.util.regex.Pattern.compile(ePattern);
        java.util.regex.Matcher m = p.matcher(email);
        return m.matches();
    }

    private void startAndShowBugReportTask(final BugReportToFileTask bugReportTask) {
        this.reportTask = bugReportTask;
        submitting.set(true);

        progressLabel.textProperty().bind(bugReportTask.messageProperty());
        submitProgressIndicator.progressProperty().bind(bugReportTask.progressProperty());


        bugReportTask.setOnSucceeded(workerStateEvent -> {
            JFDialogs.create()
                    .owner(submitButton.getScene().getWindow())
                    .message(bugReportTask.getValue())
                    .masthead("Bug report result")
                    .title(null)
                    .showInformation();

            BugReportDialog.this.reportTask = null;
            close();
        });

        bugReportTask.setOnFailed(workerStateEvent -> {
            JFDialogs.create()
                    .owner(submitButton.getScene().getWindow())
                    .message("An error occurred while sending the bug report. Are you connected to the internet?")
                    .masthead("Error while sending bug report")
                    .title("Error while sending bug report")
                    .showException(bugReportTask.getException());

            submitting.set(false);
            progressLabel.textProperty().unbind();
            progressLabel.setText("Processing bug report failed");
            submitProgressIndicator.progressProperty().unbind();
            BugReportDialog.this.reportTask = null;
        });

        new Thread(bugReportTask).start();
    }

    private void close() {
        Stage stage = (Stage) cancelButton.getScene().getWindow();
        if (stage != null)
            stage.close();
    }

    @FXML
    private void onCancelAction() {
        if (reportTask != null) {
            reportTask.cancel();
        }
        close();
    }

    @Override
    public void registerScene(Scene scene) {
        scene.getAccelerators().put(new KeyCodeCombination(KeyCode.F10), () -> saveToFileButton.setVisible(!saveToFileButton.isVisible()));
    }

    @FXML
    private void fetchScreenshot() {
        if (systemClipboard.hasImage()) {
            screenshotImageView.setImage(systemClipboard.getImage());

        } else {
            String osSpecificHelpmessage = "";

            OSDetector.OS currentOS = OSDetector.findOS();
            if (currentOS == OSDetector.OS.WIN) {
                osSpecificHelpmessage =
                        "\t- Press 'Print Screen'/ 'Prnt Scrn' / 'PrtScn',\n" +
                                "\t- Press ALT+'Print Screen' if you only want to make a picture of the active window.";
            } else if (currentOS == OSDetector.OS.MAC) {
                osSpecificHelpmessage =
                        "\t- Press Command-Control-Shift-3 for the whole screen,\n" +
                                "\t- Press Command-Control-Shift-4 if you want to select a specific area of your screen first,\n" +
                                "\t- Press Command-Control-Shift-4 followed by Space if you want to select a specific window.";
            } else {
                osSpecificHelpmessage = "\t- On Ubuntu: press CTRL+'PrtScn'\n" +
                        "\t- On KDE: press 'PrtScn'\n" +
                        "\t- On Gnome: execute 'gnome-screenshot -a -c'";
            }


            JFDialogs.create()
                    .message("The clipboard doesn't contain an image at the moment. " +
                            "To include a screenshot in the bugreport, make a screenshot of the relevant windows, then press the button in the Bug Report dialog.\n\nTo make a screenshot and store it on the clipboard: \n" + osSpecificHelpmessage)
                    .masthead("Fetching screenshot from clipboard failed")
                    .showError();
        }


    }
}
